package com.wechi.adweb.bridge.google.analytics.dto.report.data;

import static com.wechi.adweb.bridge.google.analytics.dto.report.ReportConstant.*;

import com.google.analytics.data.v1beta.Row;
import com.google.common.collect.Lists;
import com.wechi.adweb.bridge.google.analytics.dto.report.ReportType;
import com.wechi.adweb.bridge.util.DateUtils;

import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * Represents a flexible GA report for {@link ReportType#ADWEB_CUSTOM_REPORT}, structured as a
 * {@link Map} collection.
 *
 * <p>This {@link Map} contains two types of key-value pairs:
 *
 * <ul>
 *   <li><strong>Dimensions:</strong>
 *       <ul>
 *         <li><strong>Key:</strong> the name of the dimension.
 *         <li><strong>Value:</strong> the value of the dimension.
 *       </ul>
 *   <li><strong>Metrics:</strong>
 *       <ul>
 *         <li><strong>Key:</strong> the name of the metric.
 *         <li><strong>Value:</strong> the value of the metric.
 *       </ul>
 * </ul>
 *
 * <p>This structure allows for flexible representation of both dimensions and metrics in custom GA
 * reports.
 *
 * @author wfansh
 */
public class CustomReportData extends LinkedHashMap<String, String> implements GAReportDataDTO {

    private CustomReportData() {}

    public static List<CustomReportData> toReport(
            List<Row> reportRows, List<String> metrics, List<String> dimensions) {
        List<CustomReportData> report = Lists.newArrayList();
        for (Row reportRow : reportRows) {
            CustomReportData reportData = new CustomReportData();
            // Dimensions.
            for (int i = 0; i < dimensions.size(); i++) {
                String dimension = dimensions.get(i);
                /**
                 * Formatted as {@link DateTimeFormatter#ISO_LOCAL_DATE}, originated from {@link
                 * DateTimeFormatter#BASIC_ISO_DATE}
                 */
                if (DIMENSION_DATE.equalsIgnoreCase(dimension)) {
                    reportData.put(
                            dimension,
                            DateUtils.format(
                                    reportRow.getDimensionValues(i).getValue(),
                                    DateTimeFormatter.BASIC_ISO_DATE,
                                    DateTimeFormatter.ISO_LOCAL_DATE));
                } else {
                    reportData.put(dimension, reportRow.getDimensionValues(i).getValue());
                }
            }

            // Metrics.
            for (int i = 0; i < metrics.size(); i++) {
                reportData.put(metrics.get(i), reportRow.getMetricValues(i).getValue());
            }

            report.add(reportData);
        }

        return report;
    }
}
