package com.wechi.adweb.bridge.google.analytics.dto.report.data;

import static com.wechi.adweb.bridge.google.analytics.dto.report.ReportConstant.*;

import com.google.analytics.data.v1beta.Row;
import com.wechi.adweb.bridge.util.DateUtils;
import com.wechi.adweb.bridge.util.NumberUtils;

import lombok.Builder;
import lombok.Data;

import java.math.BigDecimal;
import java.time.format.DateTimeFormatter;
import java.util.List;

/**
 * @author wfansh
 */
@Data
@Builder
public class DateViewData implements GAReportDataDTO {

    /**
     * Formatted as {@link DateTimeFormatter#ISO_LOCAL_DATE}, originated from {@link
     * DateTimeFormatter#BASIC_ISO_DATE}
     */
    private String date;

    private int totalUsers;

    private int activeUsers;

    private int pageViews;

    private BigDecimal bounceRate;

    private BigDecimal avgSessionDuration;

    public static List<DateViewData> toReport(
            List<Row> reportRows, List<String> metrics, List<String> dimensions) {
        int indexDate = dimensions.indexOf(DIMENSION_DATE);
        int indexTotalUsers = metrics.indexOf(METRIC_TOTAL_USERS);
        int indexActiveUsers = metrics.indexOf(METRIC_ACTIVE_USERS);
        int indexPageViews = metrics.indexOf(METRIC_SCREEN_PAGE_VIEWS);
        int indexBounceRate = metrics.indexOf(METRIC_BOUNCE_RATE);
        int indexAvgSessionDuration = metrics.indexOf(METRIC_AVG_SESSION_DURATION);

        return reportRows.stream()
                .map(
                        row ->
                                DateViewData.builder()
                                        .date(
                                                DateUtils.format(
                                                        row.getDimensionValues(indexDate)
                                                                .getValue(),
                                                        DateTimeFormatter.BASIC_ISO_DATE,
                                                        DateTimeFormatter.ISO_LOCAL_DATE))
                                        .totalUsers(
                                                Integer.parseInt(
                                                        row.getMetricValues(indexTotalUsers)
                                                                .getValue()))
                                        .activeUsers(
                                                Integer.parseInt(
                                                        row.getMetricValues(indexActiveUsers)
                                                                .getValue()))
                                        .pageViews(
                                                Integer.parseInt(
                                                        row.getMetricValues(indexPageViews)
                                                                .getValue()))
                                        .bounceRate(
                                                NumberUtils.formatDecimal(
                                                        Double.parseDouble(
                                                                row.getMetricValues(indexBounceRate)
                                                                        .getValue()),
                                                        4))
                                        .avgSessionDuration(
                                                NumberUtils.formatDecimal(
                                                        Double.parseDouble(
                                                                row.getMetricValues(
                                                                                indexAvgSessionDuration)
                                                                        .getValue()),
                                                        2))
                                        .build())
                .toList();
    }
}
